/*
 * Copyright © 2011 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation. 
 * Oracle and Java are trademarks or registered trademarks of Oracle and/or its
 * affiliates. Other product and company names mentioned herein may be trademarks
 * or trade names of their respective owners. 
 * See LICENSE.TXT for license information.
 */
package com.nokia.example.attractions.location;

import com.nokia.example.attractions.utils.Sleeper;
import javax.microedition.location.Criteria;
import javax.microedition.location.Location;
import javax.microedition.location.LocationException;
import javax.microedition.location.LocationProvider;
import javax.microedition.location.QualifiedCoordinates;

/**
 * Location finder which uses GPS.
 */
class GpsLocationFinder
    extends LocationFinder {

    private static final int INTERVAL = 5 * 60000;
    private Criteria criteria;
    private volatile boolean running = false;
    private LocationProvider lp = null;
    private final Sleeper sleeper = new Sleeper();
    private long positionUpdated = -1;
    private static boolean hasLocationPermission = true;

    public GpsLocationFinder() {
        criteria = new Criteria();
        criteria.setCostAllowed(true);
        criteria.setPreferredPowerConsumption(Criteria.NO_REQUIREMENT);
        criteria.setSpeedAndCourseRequired(false);
        criteria.setAltitudeRequired(false);
        criteria.setAddressInfoRequired(false);
    }

    /**
     * @see LocationFinder#start(int) 
     */
    public final void start() {
        if (!hasLocationPermission) {
            return;
        }

        if (running) {
            return;
        }
        running = true;

        new Thread() {

            public void run() {
                while (running) {
                    requestLocation();
                    sleeper.sleep(getInterval() / 3);
                }
            }
        }.start();
    }

    /**
     * @see LocationFinder#quit() 
     */
    public final void quit() {
        running = false;
        if (lp != null) {
            lp.reset();
        }
    }

    /**
     * @return interval for polling the location provider
     */
    protected int getInterval() {
        return INTERVAL;
    }

    private synchronized void requestLocation() {
        if (System.currentTimeMillis() < positionUpdated + getInterval()) {
            return;
        }

        try {
            if (lp == null) {
                try {
                    lp = getLocationProvider();
                }
                catch (LocationException le) {
                    System.out.println("LOCATION ERROR:" + le.getMessage());
                }
            }
            if (lp == null) {
                hasLocationPermission = false;
                running = false;
                return;
            }

            updateLocation(lp.getLocation(60));
        }
        catch (LocationException e) {
            System.out.println("LOCATION ERROR:" + e.getMessage());
        }
        catch (InterruptedException e) {
            System.out.println("INTERRUPTED ERROR:" + e.getMessage());
        }
        catch (SecurityException e) {
            hasLocationPermission = false;
            running = false;
        }
    }

    /**
     * Get location provider. Sub-classes should override this to return 
     * location provider with different criterias.
     * @return location provider
     * @throws LocationException 
     */
    protected LocationProvider getLocationProvider()
        throws LocationException {
        return LocationProvider.getInstance(criteria);
    }

    private synchronized void updateLocation(Location lctn) {
        QualifiedCoordinates coordinates = lctn.getQualifiedCoordinates();
        if (coordinates != null) {
            positionUpdated = System.currentTimeMillis();
            double lat = coordinates.getLatitude();
            double lon = coordinates.getLongitude();
            int accuracy = (int) coordinates.getHorizontalAccuracy();
            accuracy = Math.max(accuracy, 10);
            listener.newLocation(lat, lon, accuracy);
        }
    }
}
